#!/usr/bin/env python
# version 2
# I created this script, following steps in "How to create a Crystal Sphere Gimp 2.10 tutorial" 
# Translated to GIMP from http://www.lunacore.com/photoshop/tutorials/tut009.htm ; by conbagui found at link: http://fav.me/dd1h7ws
# 
# It Creates a 1000 px crystal sphere.
# It have the option for a patterns

# author: Pocholo
# date: 5/23/20

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.


from gimpfu import *

def pm_glass_sphere(scale, pattern, shadowlayer):
	
	imgSize = 1000
	
	#Add a new layer and a pattern background
	img = pdb.gimp_image_new(imgSize, imgSize, RGB_IMAGE)
	backLayer = pdb.gimp_layer_new(img, imgSize, imgSize, RGB_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(img, backLayer, None, 0)
	pdb.gimp_layer_add_alpha(backLayer)
	pdb.gimp_context_set_pattern(pattern)
	pdb.gimp_drawable_fill(backLayer, FILL_PATTERN)
	
	disabled = pdb.gimp_image_undo_disable(img)
	#Create a the sphere layer	
	sphereLayer = pdb.gimp_layer_new(img, imgSize, imgSize, RGB_IMAGE, "Sphere", 0, LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(img, sphereLayer, None, 0)
	pdb.gimp_layer_add_alpha(sphereLayer)
	pdb.gimp_drawable_fill(sphereLayer, FILL_TRANSPARENT)
	
	#Create the shape
	pdb.gimp_image_select_ellipse(img, 2, 300, 300, 400, 400)
	pdb.gimp_context_set_foreground((202, 202, 202))
	pdb.gimp_edit_fill(sphereLayer, 0)
	pdb.plug_in_sel2path(img, sphereLayer)
	
	#Add a new layer and apply gradient
	gradLayer = pdb.gimp_layer_new(img, imgSize, imgSize, RGB_IMAGE, "Gradient", 100, LAYER_MODE_SOFTLIGHT)
	pdb.gimp_image_insert_layer(img, gradLayer, None, 0)
	pdb.gimp_layer_add_alpha(gradLayer)
	pdb.gimp_context_set_foreground((000, 000, 000))
	pdb.gimp_context_set_background((255, 255, 255))
	pdb.gimp_drawable_fill(gradLayer, FILL_TRANSPARENT)
	pdb.gimp_context_set_gradient_fg_bg_rgb()
	pdb.gimp_drawable_edit_gradient_fill(gradLayer, 0, 0, TRUE, 3, 0.20, TRUE, 500, 500, 385, 665)
	
	#Create inner glass layer
	innerglassLayer = pdb.gimp_layer_new(img, imgSize, imgSize, RGB_IMAGE, "Inner glass", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(img, innerglassLayer, None, 0)
	pdb.gimp_layer_add_alpha(innerglassLayer)
	pdb.gimp_drawable_fill(innerglassLayer, FILL_TRANSPARENT)
	pdb.gimp_context_set_foreground((255, 255, 255))
	pdb.gimp_edit_fill(innerglassLayer, 0)
		
	#Create the inner glass
	name = pdb.gimp_context_get_brush()
	pdb.gimp_context_set_brush("2. Hardness 100")
	pdb.gimp_context_set_brush_size(425)
	pdb.gimp_eraser_default(innerglassLayer, 2, [510, 480])
	pdb.gimp_selection_none(img)
	pdb.plug_in_gauss(img, innerglassLayer, 100, 100, 1)

	#Create a copy layer
	non_empty = pdb.gimp_edit_copy(innerglassLayer)
	layer = pdb.gimp_layer_new(img, imgSize, imgSize, RGB_IMAGE, "Inner glass copy", 100, LAYER_MODE_SCREEN)
	pdb.gimp_image_add_layer(img, layer, 0)
	pdb.gimp_layer_add_alpha(layer)
	pdb.gimp_drawable_fill(layer, FILL_TRANSPARENT)
	floating_sel = pdb.gimp_edit_paste(layer, FALSE)
	pdb.gimp_floating_sel_anchor(floating_sel)
	
	#Create inner shadow
	vectors = pdb.gimp_image_get_active_vectors(img)
	pdb.gimp_context_set_stroke_method(STROKE_LINE)
	pdb.gimp_context_set_line_width(4)
	pdb.gimp_context_set_foreground((0, 0, 0))
	pdb.gimp_edit_stroke_vectors(innerglassLayer, vectors)
	pdb.plug_in_gauss(img, innerglassLayer, 10, 10, 1)
	pdb.gimp_image_select_item(img, 2, vectors)
	pdb.gimp_selection_invert(img)
	pdb.gimp_edit_clear(innerglassLayer)
	pdb.gimp_selection_none(img)
	
	#Create light reflection layer
	reflectionLayer = pdb.gimp_layer_new(img, imgSize, imgSize, RGB_IMAGE, "Light reflection", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(img, reflectionLayer, None, 0)
	pdb.gimp_layer_add_alpha(reflectionLayer)
	pdb.gimp_drawable_fill(reflectionLayer, FILL_TRANSPARENT)
		
	#Create light reflection
	name = pdb.gimp_context_get_brush()
	pdb.gimp_context_set_brush("2. Hardness 050")
	pdb.gimp_context_set_brush_size(60)
	pdb.gimp_context_set_foreground((255, 255, 255))
	pdb.gimp_paintbrush_default(reflectionLayer, 2, [560, 400])
	pdb.gimp_selection_none(img)
	
	#Create the shadow layer and the shadow
	if shadowlayer == True:
		pdb.gimp_image_set_active_layer(img, sphereLayer)
		pdb.script_fu_perspective_shadow(img, sphereLayer, 140, 2.7, 0.6, 7, (0, 0, 0), 90, 1, TRUE)
		shadowlayer = pdb.gimp_image_merge_down(img, sphereLayer, CLIP_TO_IMAGE)
		name = pdb.gimp_context_get_brush()
		pdb.gimp_context_set_brush("2. Hardness 075")
		pdb.gimp_context_set_brush_size(400)
		pdb.gimp_context_set_brush_aspect_ratio(1.50)
		pdb.gimp_context_set_brush_angle(10.00)
		pdb.gimp_context_set_foreground((255, 255, 255))
		pdb.gimp_eraser_default(shadowlayer, 2, [410, 620])
		pdb.gimp_layer_set_mode(shadowlayer, LAYER_MODE_GRAIN_MERGE)
	
	#Use G"MIC plugin to create a lens distortion
	pdb.gimp_image_set_active_layer(img, gradLayer)
	pdb.gimp_image_select_item(img, 2, gradLayer)
	pdb.gimp_image_set_active_layer(img, backLayer)
	pdb.plug_in_gmic_qt(img, backLayer, 1, 0, 
		"-v - -fx_distort_lens 0.1,0,0.18,50,50,0")
	pdb.plug_in_gmic_qt(img, backLayer, 1, 0, 
		"-v - -fx_distort_lens 0.1,0,0.18,50,50,0")
	
	if shadowlayer == True:
		pdb.gimp_image_set_active_layer(img, shadowlayer)
		pdb.plug_in_gmic_qt(img, shadowlayer, 1, 0, 
			"-v - -fx_distort_lens 0.1,0,0.16,50,50,0")
	pdb.gimp_selection_none(img)
	
	#Merge layers
	
	pdb.gimp_image_set_active_layer(img, backLayer)
	
	pdb.gimp_image_scale(img, scale,scale)
	
	enabled = pdb.gimp_image_undo_enable(img)
	display = pdb.gimp_display_new(img)
	

register(
	"pm_glass_sphere",
	"Creates a glass sphere",
	"Creates a glass sphere",
	"Pocholo",
	"Pocholo",
	"2020",
	"Glass sphere",
	"",
[
	(PF_SPINNER, "imgSize", "Scale", 1000, (1000, 1500, 1)),
	(PF_PATTERN, "pattern", "Pattern", None),
	(PF_BOOL, "shadowLayer", "Shadow?", TRUE),
],
[],
pm_glass_sphere, menu="<Image>/Python-Fu/Glass sphere")

main()	